import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import {
  Document,
  AlignmentType,
  Packer,
  Paragraph,
  Table,
  TableCell,
  TableRow,
  VerticalAlign,
  TextRun,
  WidthType,
  PageOrientation
} from 'docx'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { exportDetailToWord, exportToWord } from './doc/RGangguan_DMAPKT_DOC'

const reportName = 'Rekapitulasi Gangguan Diselesaikan Mobile APKT'
const fontSize = 4
const detailFontSize = 3

const formatData = (rawData: any) => {
  const formattedData: any = []
  const grandTotal: any = {
    total_laporan: 0,
    jumlah_e_laporan: 0,
    total_i_laporan: 0,
    total_m_laporan: 0,
    total_q_laporan: 0,
    total_u_laporan: 0
  }

  for (let i = 0; i < rawData.length; i++) {
    formattedData.push([
      { content: i + 1, styles: { halign: 'right' } },
      rawData[i].nama_regu,
      rawData[i].kode_regu,
      formatNumber(rawData[i].total),
      formatNumber(rawData[i].total_selesai),
      formatPercentage(rawData[i].persen_selesai),
      formatNumber(rawData[i].total_inproses),
      formatPercentage(rawData[i].persen_inproses),
      formatNumber(rawData[i].total_mobile_nonmarking),
      formatPercentage(0),
      formatNumber(rawData[i].avg_durasi_response_time_mobile_nonmarking),
      formatNumber(rawData[i].avg_durasi_recovery_time_mobile_nonmarking),
      formatNumber(rawData[i].total_mobile_marking),
      formatPercentage(0),
      formatNumber(rawData[i].avg_durasi_response_time_mobile_marking),
      formatNumber(rawData[i].avg_durasi_recovery_time_mobile_marking),
      formatNumber(rawData[i].total_nonmobile_nonmarking),
      formatPercentage(0),
      formatNumber(rawData[i].avg_durasi_response_time_nonmobile_nonmarking),
      formatNumber(rawData[i].avg_durasi_recovery_time_nonmobile_nonmarking),
      formatNumber(rawData[i].total_nonmobile_marking),
      formatPercentage(0),
      formatNumber(rawData[i].avg_durasi_response_time_nonmobile_marking),
      formatNumber(rawData[i].avg_durasi_recovery_time_nonmobile_marking)
    ])

    grandTotal.total_laporan += rawData[i].total
    grandTotal.jumlah_e_laporan += rawData[i].total_selesai
    grandTotal.total_i_laporan += rawData[i].total_mobile_nonmarking
    grandTotal.total_m_laporan += rawData[i].total_mobile_marking
    grandTotal.total_q_laporan += rawData[i].total_nonmobile_nonmarking
    grandTotal.total_u_laporan += rawData[i].total_nonmobile_marking
  }

  formattedData.push([
    { content: 'TOTAL', colSpan: 3, styles: { fontStyle: 'bold' } },
    formatNumber(grandTotal.total_laporan),
    formatNumber(grandTotal.jumlah_e_laporan),
    '',
    '',
    '',
    formatNumber(grandTotal.total_i_laporan),
    '',
    '',
    '',
    formatNumber(grandTotal.total_m_laporan),
    '',
    '',
    '',
    formatNumber(grandTotal.total_q_laporan),
    '',
    '',
    '',
    formatNumber(grandTotal.total_u_laporan),
    '',
    '',
    ''
  ])

  return formattedData
}

const formatMetaData = (reportMeta: any) => {
  const periode = reportMeta.periode ? reportMeta.periode.split(' s/d ') : ''

  let dateFromFormat = ''
  let dateToFormat = ''
  let dayTo = ''

  if (periode != '') {
    const dateFrom = new Date(periode[0].split('-').reverse().join('-'))
    const dateTo = new Date(periode[1].split('-').reverse().join('-'))

    dateFromFormat = `${dateFrom.getDate()}-${dateFrom.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateFrom.getFullYear()}`

    dateToFormat = `${dateTo.getDate()}-${dateTo.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateTo.getFullYear()}`

    dayTo = dateTo.toLocaleString('id-ID', { weekday: 'long' })
  }

  return { dateFromFormat, dateToFormat, dayTo }
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const data = formatData(rawData)
  const meta = formatMetaData(reportMeta)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'POSKO',
        ':',
        reportMeta.posko ? reportMeta.posko.name.toUpperCase() : 'Semua Posko'.toUpperCase()
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10
  })

  autoTable(doc, {
    head: [
      [`${reportName}`.toUpperCase()],
      [`PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 23
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'No',
          rowSpan: 6
        },
        {
          content: 'Nama Regu',
          rowSpan: 6
        },
        {
          content: 'Kode Regu',
          rowSpan: 6
        },
        {
          content: 'Laporan',
          colSpan: 5
        },
        {
          content: 'Penyelesaian',
          colSpan: 16
        }
      ],
      [
        'Total',
        {
          content: 'Sudah Selesai',
          colSpan: 2
        },
        {
          content: 'Belum Selesai',
          colSpan: 2
        },
        {
          content: 'Mobile',
          colSpan: 8
        },
        {
          content: 'Non Mobile',
          colSpan: 8
        }
      ],
      [
        {
          content: 'd=(e+g)',
          rowSpan: 4
        },
        'Jml',
        '%',
        'Jml',
        '%',
        {
          content: 'Non Marking',
          colSpan: 4
        },
        {
          content: 'Marking',
          colSpan: 4
        },
        {
          content: 'Non Marking',
          colSpan: 4
        },
        {
          content: 'Marking',
          colSpan: 4
        }
      ],
      [
        {
          content: 'e',
          rowSpan: 3
        },
        {
          content: 'f=(e/d)*100',
          rowSpan: 3
        },
        {
          content: 'g',
          rowSpan: 3
        },
        {
          content: 'h=(g/d)*100',
          rowSpan: 3
        },
        {
          content: 'Total',
          colSpan: 2
        },
        {
          content: 'Waktu Rata-Rata',
          colSpan: 2
        },
        {
          content: 'Total',
          colSpan: 2
        },
        {
          content: 'Waktu Rata-Rata',
          colSpan: 2
        },
        {
          content: 'Total',
          colSpan: 2
        },
        {
          content: 'Waktu Rata-Rata',
          colSpan: 2
        },
        {
          content: 'Total',
          colSpan: 2
        },
        {
          content: 'Waktu Rata-Rata',
          colSpan: 2
        }
      ],
      [
        'Laporan',
        '%',
        'Response',
        'Recovery',
        'Laporan',
        '%',
        'Response',
        'Recovery',
        'Laporan',
        '%',
        'Response',
        'Recovery',
        'Laporan',
        '%',
        'Response',
        'Recovery'
      ],
      [
        'i',
        'j=(i/d)*100',
        'k',
        'l',
        'm',
        'n=(m/d)*100',
        'o',
        'p',
        'q',
        'r=(q/d)*100',
        's',
        't',
        'u',
        'v=(u/d)*100',
        'w',
        'x'
      ]
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'TOTAL') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }

      if (data.cell.text[0] === 'GRAND TOTAL') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 30
  })

  autoTable(doc, {
    head: [
      [`${meta.dayTo}, ${meta.dateToFormat}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const exportDetailToPDF = (reportMeta: any, rawData: any) => {
  const meta = formatMetaData(reportMeta)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [['PT. PLN(Persero)']],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [`Daftar Detail ${reportName}`.toUpperCase()],
      [`PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`]
    ],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 18,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [
        'No',
        'No Laporan',
        'Tgl Lapor',
        'Tgl Datang',
        'Tgl Nyala',
        'Durasi Response Time',
        'Durasi Recovery Time',
        'Status',
        'Referensi Marking',
        'IDPEL/NO METER',
        'Nama Pelapor',
        'Alamat Pelapor',
        'No Telp Pelapor',
        'Keterangan Pelapor',
        'Posko',
        'Tindakan',
        'Penyebab'
      ]
    ],
    body: rawData.map((item: any, i: any) => [
      { content: i + 1, styles: { halign: 'right' } },
      item.no_laporan,
      item.waktu_lapor,
      item.waktu_response,
      item.waktu_recovery,
      item.durasi_response_time,
      item.durasi_recovery_time,
      item.status_akhir,
      item.referensi_marking,
      item.idpel_nometer,
      item.nama_pelapor,
      item.alamat_pelapor,
      item.no_telp_pelapor,
      item.keterangan_pelapor,
      item.nama_posko,
      item.tindakan,
      item.penyebab
    ]),
    styles: {
      fontSize: detailFontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }
    },
    startY: 24,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [`${meta.dayTo}, ${meta.dateToFormat}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  doc.save(`Laporan Detail ${reportName}.pdf`, { returnPromise: true }).then(() => {
    console.log('pdf berhasil disimpan')
  })
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const meta = formatMetaData(reportMeta)
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${reportMeta.uid
      ? reportMeta.uid.name.toUpperCase()
      : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${reportMeta.up3
      ? reportMeta.up3.name.toUpperCase()
      : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `POSKO : ${reportMeta.posko ? reportMeta.posko.name.toUpperCase() : 'Semua Posko'.toUpperCase()
    }`
  )

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(
    worksheet,
    8,
    1,
    `PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`,
    true
  )

  worksheet.mergeCells('A1:X1')
  worksheet.mergeCells('A2:X2')
  worksheet.mergeCells('A3:X3')
  worksheet.mergeCells('A4:X4')
  worksheet.mergeCells('A7:X7')
  worksheet.mergeCells('A8:X8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportDetailToXLSX = (reportMeta: any, e: any) => {
  const meta = formatMetaData(reportMeta)
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`Detail ${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(worksheet, 3, 1, `Daftar Detail ${reportName}`.toUpperCase(), true)
  setHeaderStyle(
    worksheet,
    4,
    1,
    `PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`,
    true
  )

  worksheet.mergeCells('A3:Q3')
  worksheet.mergeCells('A4:Q4')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 6, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(
        new Blob([buffer], { type: 'application/octet-stream' }),
        `Laporan Detail ${reportName}.xlsx`
      )
    })
  })

  e.cancel = true
}
const exportToDOCX = (reportMeta: any, rawData: any) => {
  exportToWord(reportMeta, formatData(rawData), `Laporan ${reportName}`, formatMetaData(reportMeta))
}
const exportDetailToDOCX = (reportMeta: any, rawData: any) => {
  exportDetailToWord(reportMeta, rawData, `Laporan Detail ${reportName}`, formatMetaData(reportMeta))
}
export { exportToPDF, exportToXLSX, exportDetailToPDF, exportDetailToXLSX, exportToDOCX, exportDetailToDOCX }
