import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { formatWaktu } from '@/components/Form/FiltersType/reference'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { getMonthName } from '@/utils/texts'
import { exportDetailToWord, exportToWord } from './doc/MonalisaGR_DispatchingTimeGangguan_DOC'

const reportName = 'Dispacthing Time (DT) Gangguan'
const fontSize = 5
const detailFontSize = 3

const getGroupParent = (reportMeta: any) => {
  if (reportMeta.ulp.id != 0) {
    return {
      parent: reportMeta.ulp.name,
      summaryName: reportMeta.ulp.name,
      summaryKey: 'nama_ulp'
    }
  }

  if (reportMeta.up3.id != 0) {
    return {
      parent: reportMeta.up3.name,
      summaryName: reportMeta.up3.name,
      summaryKey: 'nama_up3'
    }
  }

  if (reportMeta.uid.id != 0) {
    return {
      parent: reportMeta.uid.name,
      summaryName: reportMeta.uid.name,
      summaryKey: 'nama_uid'
    }
  }

  if (reportMeta.regional.id != 0) {
    return {
      parent: reportMeta.regional.name,
      summaryName: reportMeta.regional.name,
      summaryKey: 'nama_regional'
    }
  }

  return ''
}

const groupingData = (data: any, reportMeta: any) => {
  const groupedData: any = {}

  if (reportMeta.regional.id == 0) {
    data.forEach((item: any) => {
      const { nama_regional } = item

      if (!groupedData[nama_regional]) {
        groupedData[nama_regional] = { data: [] }
      }

      groupedData[nama_regional].data.push(item)
    })

    for (const regional in groupedData) {
      const data = groupedData[regional].data

      data.forEach((item: any) => {
        if (!groupedData[regional].summary) {
          groupedData[regional].summary = {
            mom_bulan_kemarin: 0,
            mom_bulan_ini: 0,
            persen_mom: [],
            yoy_tahun_kemarin: 0,
            yoy_tahun_ini: 0,
            persen_yoy: []
          }
        }

        groupedData[regional].summary.mom_bulan_kemarin += item.mom_bulan_kemarin
        groupedData[regional].summary.mom_bulan_ini += item.mom_bulan_ini
        groupedData[regional].summary.persen_mom.push(item.persen_mom)
        groupedData[regional].summary.yoy_tahun_kemarin += item.yoy_tahun_kemarin
        groupedData[regional].summary.yoy_tahun_ini += item.yoy_tahun_ini
        groupedData[regional].summary.persen_yoy.push(item.persen_yoy)
      })
    }

    return groupedData
  } else {
    const groupParent = getGroupParent(reportMeta)

    if (groupParent === '') {
      return {
        data,
        parent: '',
        summaryName: ''
      }
    }

    data.forEach((item: any) => {
      const group = item[groupParent.summaryKey]

      if (!groupedData[group]) {
        groupedData[group] = []
      }

      groupedData[group].push(item)
    })

    return {
      data: groupedData,
      parent: groupParent.parent,
      summaryName: groupParent.summaryName
    }
  }
}

const formatData = (rawData: any, reportMeta: any) => {
  const formattedData: any = []

  const total: any = {
    mom_bulan_kemarin: [],
    mom_bulan_ini: [],
    yoy_tahun_kemarin: [],
    yoy_tahun_ini: []
  }

  if (reportMeta.regional.id == 0) {
    const data = groupingData(rawData, reportMeta)
    const formattedData: any = []

    formattedData.push([{ content: 'NASIONAL', colSpan: 7, styles: { fontStyle: 'bold' } }])

    let no = 1
    for (const regional in data) {
      const summary = data[regional].summary

      formattedData.push([
        { content: no++, styles: { halign: 'right' } },
        regional,
        formatNumber(summary.mom_bulan_kemarin),
        formatNumber(summary.mom_bulan_ini),
        formatPercentage(
          !summary.mom_bulan_ini || !summary.mom_bulan_kemarin
            ? '0%'
            : ((summary.mom_bulan_kemarin - summary.mom_bulan_ini) / summary.mom_bulan_kemarin) *
                100
        ),
        formatNumber(summary.yoy_tahun_kemarin),
        formatNumber(summary.yoy_tahun_ini),
        formatPercentage(
          !summary.yoy_tahun_ini || !summary.yoy_tahun_kemarin
            ? '0%'
            : ((summary.yoy_tahun_kemarin - summary.yoy_tahun_ini) / summary.yoy_tahun_kemarin) *
                100
        )
      ])

      total.mom_bulan_kemarin.push(summary.mom_bulan_kemarin)
      total.mom_bulan_ini.push(summary.mom_bulan_ini)
      total.yoy_tahun_kemarin.push(summary.yoy_tahun_kemarin)
      total.yoy_tahun_ini.push(summary.yoy_tahun_ini)
    }

    const totalMoMBulanKemarin =
      total.mom_bulan_kemarin.reduce((a: number, b: number) => a + b, 0) /
      total.mom_bulan_kemarin.length
    const totalMoMBulanIni =
      total.mom_bulan_ini.reduce((a: number, b: number) => a + b, 0) / total.mom_bulan_ini.length
    const totalYoYTahunKemarin =
      total.yoy_tahun_kemarin.reduce((a: number, b: number) => a + b, 0) /
      total.yoy_tahun_kemarin.length
    const totalYoYTahunIni =
      total.yoy_tahun_ini.reduce((a: number, b: number) => a + b, 0) / total.yoy_tahun_ini.length

    formattedData.push([
      { content: 'TOTAL RATA-RATA', colSpan: 2, styles: { fontStyle: 'bold' } },
      formatNumber(totalMoMBulanKemarin),
      formatNumber(totalMoMBulanIni),
      formatPercentage(
        !totalMoMBulanIni || !totalMoMBulanKemarin
          ? '0%'
          : ((totalMoMBulanKemarin - totalMoMBulanIni) / totalMoMBulanKemarin) * 100
      ),
      formatNumber(totalYoYTahunKemarin),
      formatNumber(totalYoYTahunIni),
      formatPercentage(
        !totalYoYTahunIni || !totalYoYTahunKemarin
          ? '0%'
          : ((totalYoYTahunKemarin - totalYoYTahunIni) / totalYoYTahunKemarin) * 100
      )
    ])

    return formattedData
  } else {
    const tempData = groupingData(rawData, reportMeta)
    const data = tempData.data[tempData.summaryName] || tempData.data
    const formattedData: any = []

    let parentName = tempData.parent
    let summaryName = tempData.summaryName ? tempData.summaryName : 'Seluruh Unit'

    data.forEach((item: any) => {
      total.mom_bulan_kemarin.push(item.mom_bulan_kemarin)
      total.mom_bulan_ini.push(item.mom_bulan_ini)
      total.yoy_tahun_kemarin.push(item.yoy_tahun_kemarin)
      total.yoy_tahun_ini.push(item.yoy_tahun_ini)
    })

    const totalMoMBulanKemarin =
      total.mom_bulan_kemarin.reduce((a: number, b: number) => a + b, 0) /
      total.mom_bulan_kemarin.length
    const totalMoMBulanIni =
      total.mom_bulan_ini.reduce((a: number, b: number) => a + b, 0) / total.mom_bulan_ini.length
    const totalYoYTahunKemarin =
      total.yoy_tahun_kemarin.reduce((a: number, b: number) => a + b, 0) /
      total.yoy_tahun_kemarin.length
    const totalYoYTahunIni =
      total.yoy_tahun_ini.reduce((a: number, b: number) => a + b, 0) / total.yoy_tahun_ini.length

    formattedData.push([{ content: parentName, colSpan: 7, styles: { fontStyle: 'bold' } }])

    const result = [
      formatNumber(totalMoMBulanKemarin),
      formatNumber(totalMoMBulanIni),
      formatPercentage(
        !totalMoMBulanIni || !totalMoMBulanKemarin
          ? '0%'
          : ((totalMoMBulanKemarin - totalMoMBulanIni) / totalMoMBulanKemarin) * 100
      ),
      formatNumber(totalYoYTahunKemarin),
      formatNumber(totalYoYTahunIni),
      formatPercentage(
        !totalYoYTahunIni || !totalYoYTahunKemarin
          ? '0%'
          : ((totalYoYTahunKemarin - totalYoYTahunIni) / totalYoYTahunKemarin) * 100
      )
    ]

    formattedData.push([{ content: '1', styles: { halign: 'right' } }, summaryName, ...result])
    formattedData.push([
      { content: 'TOTAL RATA-RATA', colSpan: 2, styles: { fontStyle: 'bold' } },
      ...result
    ])

    return formattedData
  }
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const data = formatData(rawData, reportMeta)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'UNIT LAYANAN PELANGGAN',
        ':',
        reportMeta.ulp
          ? reportMeta.ulp.name.toUpperCase()
          : 'Semua Unit Layanan Pelanggan'.toUpperCase()
      ],
      [
        'REGIONAL',
        ':',
        reportMeta.regional
          ? reportMeta.regional.name.toUpperCase()
          : 'Semua Regional'.toUpperCase()
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10
  })

  autoTable(doc, {
    head: [[`${reportName}`.toUpperCase()], [reportMeta.periode]],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 23
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'No',
          rowSpan: 3
        },
        {
          content: 'Nama Unit',
          rowSpan: 3
        },
        {
          content: 'Jumlah Dispacthing Time Gangguan',
          colSpan: 6
        }
      ],
      [
        {
          content: 'MOM',
          colSpan: 3
        },
        {
          content: 'YOY',
          colSpan: 3
        }
      ],
      [
        `${getMonthName(reportMeta.lastMonth)} ${reportMeta.lastYearMoM}`,
        `${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%',
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.lastYear}`,
        `s.d ${getMonthName(reportMeta.currentMonth)} ${reportMeta.currentYear}`,
        '%'
      ]
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'TOTAL RATA-RATA') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 30
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const exportDetailToPDF = (reportMeta: any, rawData: any) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [['PT. PLN(Persero)']],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10,
    margin: 5
  })

  autoTable(doc, {
    head: [[`Daftar Detail ${reportName}`.toUpperCase()], [reportMeta.periode]],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 18,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [
        'No',
        'ULP',
        'No Laporan',
        'Tgl/ Jam Lapor',
        'Tgl/ Jam Datang',
        'Tgl/ Jam Nyala',
        'Durasi Response Time',
        'Durasi Recovery Time',
        'Durasi Penugasan Regu',
        'Durasi Perjalanan Regu',
        'Dispatch Oleh',
        'IDPEL/NO METER',
        'Nama Pelapor',
        'Alamat Pelapor',
        'No Telp Pelapor',
        'Keterangan Pelapor',
        'Sumber Lapor',
        'Diselesaikan Oleh',
        'Status',
        'Referensi Marking',
        'Kode Gangguan',
        'Jenis Gangguan',
        'Tindakan'
      ]
    ],
    body: rawData.map((item: any, i: any) => [
      { content: i + 1, styles: { halign: 'right' } },
      item.nama_ulp,
      item.no_laporan,
      item.waktu_lapor,
      item.waktu_response,
      item.waktu_recovery,
      item.durasi_response_time ? formatWaktu(item.durasi_response_time) : '',
      item.durasi_recovery_time ? formatWaktu(item.durasi_recovery_time) : '',
      '',
      '',
      '',
      item.idpel_nometer,
      item.nama_pelapor,
      item.alamat_pelapor,
      item.no_telp_pelapor,
      item.keterangan_pelapor,
      '',
      '',
      item.status_akhir,
      '',
      '',
      '',
      ''
    ]),
    styles: {
      fontSize: detailFontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      cellWidth: 'wrap',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }
    },
    startY: 24,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  doc.save(`Laporan Detail ${reportName}.pdf`, { returnPromise: true }).then(() => {
    console.log('pdf berhasil disimpan')
  })
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${
      reportMeta.uid
        ? reportMeta.uid.name.toUpperCase()
        : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${
      reportMeta.up3
        ? reportMeta.up3.name.toUpperCase()
        : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `UNIT LAYANAN PELANGGAN : ${
      reportMeta.ulp
        ? reportMeta.ulp.name.toUpperCase()
        : 'Semua Unit Layanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    5,
    1,
    `REGIONAL : ${
      reportMeta.regional ? reportMeta.regional.name.toUpperCase() : 'Semua Regional'.toUpperCase()
    }`
  )

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(worksheet, 8, 1, reportMeta.periode, true)

  worksheet.mergeCells('A1:G1')
  worksheet.mergeCells('A2:G2')
  worksheet.mergeCells('A3:G3')
  worksheet.mergeCells('A4:G4')
  worksheet.mergeCells('A5:G5')
  worksheet.mergeCells('A7:G7')
  worksheet.mergeCells('A8:G8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportDetailToXLSX = (reportMeta: any, e: any) => {
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`Detail ${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(worksheet, 3, 1, `Daftar Detail ${reportName}`.toUpperCase(), true)
  setHeaderStyle(worksheet, 4, 1, reportMeta.periode, true)

  worksheet.mergeCells('A1:W1')
  worksheet.mergeCells('A3:W3')
  worksheet.mergeCells('A4:W4')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 6, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(
        new Blob([buffer], { type: 'application/octet-stream' }),
        `Laporan Detail ${reportName}.xlsx`
      )
    })
  })

  e.cancel = true
}

const exportToDOCX = (reportMeta: any, rawData: any) => {
  exportToWord(reportMeta, formatData(rawData, reportMeta), `Laporan ${reportName}`, null)
}
const exportDetailToDOCX = (reportMeta: any, rawData: any) => {
  exportDetailToWord(reportMeta, rawData, `Laporan Detail ${reportName}`, null)
}
export {
  exportToPDF,
  exportDetailToPDF,
  exportToXLSX,
  exportDetailToXLSX,
  exportToDOCX,
  exportDetailToDOCX
}
